﻿using System.Collections.Generic;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Data;
using System.Collections;
using System;

namespace Dx.Controls
{
    public class FloatingControl : Selector
    {
        private const double offset = 25;
        private const double initialOffset = 25;

        Dictionary<FloatingControlItem, Rect> floatingPositionDic = new Dictionary<FloatingControlItem, Rect>();

        private List<FloatingControlItem> minimizedItem = new List<FloatingControlItem>();

        static FloatingControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(FloatingControl), new FrameworkPropertyMetadata(typeof(FloatingControl)));
        }

        public FloatingControl()
        {
            this.CommandBindings.Add(new CommandBinding(FloatingControlItem.CloseCommand, CloseFloatingItem));
            this.CommandBindings.Add(new CommandBinding(FloatingControlItem.MinimizeCommand, MinimizeFloatingItem));
        }

        private void MinimizeFloatingItem(object sender, ExecutedRoutedEventArgs e)
        {
            FloatingControlItem floatingItem = (e.OriginalSource as DependencyObject).FindParentInVisualTreeNode<FloatingControlItem>(int.MaxValue);
            if (floatingItem == null) return;
            if (!floatingItem.IsMinimized)
            {//Save Position and Minimize
                floatingPositionDic.Add(floatingItem, new Rect(Canvas.GetLeft(floatingItem), Canvas.GetTop(floatingItem), floatingItem.ActualWidth, floatingItem.ActualHeight));
                MinimizeControl(floatingItem);
                floatingItem.IsMinimized = true;
            }
            else
            {
                RecoverFloatingControl(floatingItem);
                floatingPositionDic.Remove(floatingItem);
                floatingItem.IsMinimized = false;
            }
            
        }

        private void RecoverFloatingControl(FloatingControlItem floatingItem)
        {
            Rect positionRect = floatingPositionDic[floatingItem];
            Canvas.SetLeft(floatingItem,positionRect.Left);
            Canvas.SetTop(floatingItem,positionRect.Top);
            floatingItem.Width = positionRect.Width;
            floatingItem.Height = positionRect.Height;
            minimizedItem.Remove(floatingItem);
        }


        private void MinimizeControl(FloatingControlItem floatingItem)
        {
            int limitInLine = (int)(this.ActualWidth / floatingItem.MinWidth);

            floatingItem.Width = floatingItem.MinWidth;
            floatingItem.Height = floatingItem.MinHeight;
            minimizedItem.Add(floatingItem);

            int leftCount = (minimizedItem.IndexOf(floatingItem)) % limitInLine;
            int topCount = (minimizedItem.IndexOf(floatingItem))/limitInLine +1;

            Canvas.SetLeft(floatingItem, floatingItem.MinWidth * leftCount);
            Canvas.SetTop(floatingItem, this.ActualHeight - (floatingItem.MinHeight * topCount));
        }

        protected override void OnItemsChanged(System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            base.OnItemsChanged(e);
            if (e.Action == NotifyCollectionChangedAction.Reset)
            {
                for (int i = 0; i < this.Items.Count; i++)
                {
                    FloatingControlItem floatingControl;
                    if (Items[i] is FloatingControlItem)
                    {
                        floatingControl = Items[i] as FloatingControlItem;
                    }
                    else
                    {
                        floatingControl= this.ItemContainerGenerator.ContainerFromItem(Items[i]) as FloatingControlItem;
                    }
                     Canvas.SetLeft(floatingControl, initialOffset + offset * i);
                     Canvas.SetTop(floatingControl, initialOffset + offset * i);
                }
            }
        }

        private void CloseFloatingItem(object sender, ExecutedRoutedEventArgs e)
        {
            FloatingControlItem floatingItem = (e.OriginalSource as DependencyObject).FindParentInVisualTreeNode<FloatingControlItem>(int.MaxValue);
            if (floatingItem == null) return;
            if (Items.Contains(floatingItem))
            {
                Items.Remove(floatingItem);
            }
            if (Items.Contains(floatingItem.Content))
            {
                IList list = ItemsSource as IList;
                if (list != null)
                {
                    list.Remove(floatingItem.Content);
                }
                else
                {
                    throw new InvalidOperationException("This type of ItmesSource cannot support Close Command");
                }
            }
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return (item is FloatingControlItem);
        }

        protected override DependencyObject GetContainerForItemOverride()
        {
            return new FloatingControlItem();
        }
    }
}
